@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/config_f.h,v 3.13 1996/04/26 19:17:30 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/config_f.h,v 3.16 1997/10/28 22:34:14 christos Exp $ */
 /*
  * config_f.h -- configure various defines for tcsh
  *
@@ -129,12 +129,19 @@
  *		This can be much slower and no memory statistics will be
  *		provided.
  */
-#if defined(PURIFY) || defined(MALLOC_TRACE)
+#if defined(PURIFY) || defined(MALLOC_TRACE) || defined(_OSD_POSIX)
 # define SYSMALLOC
 #else
 # undef SYSMALLOC
 #endif
 
+/*
+ * USE_ACCESS	Use access(2) rather than stat(2) when POSIX is defined.
+ *		POSIX says to use stat, but stat(2) is less accurate
+ *		than access(2) for determining file access.
+ */
+#undef USE_ACCESS
+
 /*
  * REMOTEHOST	Try to determine the remote host that we logged in from
  *		using first getpeername, and then the utmp file. If
@@ -151,7 +158,7 @@
 # ifndef __GNUC__
 #  define RCSID(id) static char *rcsid = (id);
 # else
-#  define RCSID(id) static char *rcsid() { return rcsid(id); }
+#  define RCSID(id) static char *rcsid(const char *a) { return rcsid(a = id); }
 # endif /* !__GNUC__ */
 #else
 # define RCSID(id)	/* Nothing */
