@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/ed.chared.c,v 3.48 1996/10/19 17:53:49 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/ed.chared.c,v 3.52 1997/10/28 22:34:15 christos Exp $ */
 /*
  * ed.chared.c: Character editing functions.
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: ed.chared.c,v 3.48 1996/10/19 17:53:49 christos Exp $")
+RCSID("$Id: ed.chared.c,v 3.52 1997/10/28 22:34:15 christos Exp $")
 
 #include "ed.h"
 #include "tw.h"
@@ -44,10 +44,10 @@ RCSID("$Id: ed.chared.c,v 3.48 1996/10/19 17:53:49 christos Exp $")
 
 /* #define SDEBUG */
 
-#define NOP    	  0x00
-#define DELETE 	  0x01
-#define INSERT 	  0x02
-#define CHANGE 	  0x04
+#define TCSHOP_NOP    	  0x00
+#define TCSHOP_DELETE 	  0x01
+#define TCSHOP_INSERT 	  0x02
+#define TCSHOP_CHANGE 	  0x04
 
 #define CHAR_FWD	0
 #define CHAR_BACK	1
@@ -62,7 +62,7 @@ RCSID("$Id: ed.chared.c,v 3.48 1996/10/19 17:53:49 christos Exp $")
 
 static Char *InsertPos = InputBuf; /* Where insertion starts */
 static Char *ActionPos = 0;	   /* Where action begins  */
-static int  ActionFlag = NOP;	   /* What delayed action to take */
+static int  ActionFlag = TCSHOP_NOP;	   /* What delayed action to take */
 /*
  * Word search state
  */
@@ -87,6 +87,7 @@ static	Char	*c_number		__P((Char *, int *, int));
 static	Char	*c_expand		__P((Char *));
 static	void	 c_excl			__P((Char *));
 static	void	 c_substitute		__P((void));
+static	void	 c_delfini		__P((void));
 static	int	 c_hmatch		__P((Char *));
 static	void	 c_hsetpat		__P((void));
 #ifdef COMMENT
@@ -102,6 +103,7 @@ static  CCRETVAL v_repeat_srch		__P((int));
 static	CCRETVAL e_inc_search		__P((int));
 static	CCRETVAL v_search		__P((int));
 static	CCRETVAL v_csearch_fwd		__P((int, int, int));
+static	CCRETVAL v_action		__P((int));
 static	CCRETVAL v_csearch_back		__P((int, int, int));
 
 static void
@@ -150,7 +152,7 @@ c_delafter(num)
     if (num > 0) {			/* if I can delete anything */
 	if (VImode) {
 	    kp = UndoBuf;		/* Set Up for VI undo command */
-	    UndoAction = INSERT;
+	    UndoAction = TCSHOP_INSERT;
 	    UndoSize = num;
 	    UndoPtr  = Cursor;
 	    for (cp = Cursor; cp <= LastChar; cp++) {
@@ -186,7 +188,7 @@ c_delbefore(num)		/* delete before dot, with bounds checking */
     if (num > 0) {			/* if I can delete anything */
 	if (VImode) {
 	    kp = UndoBuf;		/* Set Up for VI undo command */
-	    UndoAction = INSERT;
+	    UndoAction = TCSHOP_INSERT;
 	    UndoSize = num;
 	    UndoPtr  = Cursor - num;
 	    for (cp = Cursor - num; cp <= LastChar; cp++) {
@@ -624,7 +626,7 @@ excl_sw:
     *LastChar = '\0';
     return(op + (bend - buf));
 excl_err:
-    Beep();
+    SoundBeep();
     return(op + 1);
 }
 
@@ -704,15 +706,15 @@ c_delfini()		/* Finish up delete action */
 {
     register int Size;
 
-    if (ActionFlag & INSERT)
+    if (ActionFlag & TCSHOP_INSERT)
 	c_alternativ_key_map(0);
 
-    ActionFlag = NOP;
+    ActionFlag = TCSHOP_NOP;
 
     if (ActionPos == 0) 
 	return;
 
-    UndoAction = INSERT;
+    UndoAction = TCSHOP_INSERT;
 
     if (Cursor > ActionPos) {
 	Size = (int) (Cursor-ActionPos);
@@ -911,7 +913,7 @@ e_inc_search(dir)
 	case F_DIGIT:
 	case F_MAGIC_SPACE:
 	    if (patlen > INBUFSIZE - 3)
-		Beep();
+		SoundBeep();
 	    else {
 		patbuf[patlen++] = ch;
 		*LastChar++ = ch;
@@ -934,7 +936,7 @@ e_inc_search(dir)
 	    if (patlen > 1)
 		done++;
 	    else 
-		Beep();
+		SoundBeep();
 	    break;
 
 	default:
@@ -952,7 +954,7 @@ e_inc_search(dir)
 			cp = c_next_word(Cursor, LastChar, 1);
 			while (Cursor < cp && *Cursor != '\n') {
 			    if (patlen > INBUFSIZE - 3) {
-				Beep();
+				SoundBeep();
 				break;
 			    }
 			    patbuf[patlen++] = *Cursor;
@@ -963,7 +965,7 @@ e_inc_search(dir)
 			Refresh();
 			break;
 		    } else if (isglob(*cp)) {
-			Beep();
+			SoundBeep();
 			break;
 		    }
 		break;
@@ -1021,7 +1023,7 @@ e_inc_search(dir)
 		}
 		patbuf[--patlen] = '\0';
 		if (ret == CC_ERROR) {
-		    Beep();
+		    SoundBeep();
 		    if (Hist_num != oldHist_num) {
 			Hist_num = oldHist_num;
 			if (c_get_histline() == CC_ERROR)
@@ -1091,7 +1093,7 @@ v_search(dir)
     for (ch = 0;ch == 0;) {
 	if (GetNextChar(&ch) != 1)
 	    return(e_send_eof(0));
-	switch (ch) {
+	switch (ASC(ch)) {
 	case 0010:	/* Delete and backspace */
 	case 0177:
 	    if (tmplen > 1) {
@@ -1110,13 +1112,18 @@ v_search(dir)
 	    break;
 
 	case 0033:	/* ESC */
+#ifndef _OSD_POSIX
 	case '\r':	/* Newline */
 	case '\n':
+#else
+	case '\012':    /* Newline */
+	case '\015':    /* Return */
+#endif
 	    break;
 
 	default:
 	    if (tmplen >= INBUFSIZE)
-		Beep();
+		SoundBeep();
 	    else {
 		tmpbuf[tmplen++] = ch;
 		*Cursor++ = ch;
@@ -1187,7 +1194,7 @@ v_cmd_mode(c)
 {
     USE(c);
     InsertPos = 0;
-    ActionFlag = NOP;	/* [Esc] cancels pending action */
+    ActionFlag = TCSHOP_NOP;	/* [Esc] cancels pending action */
     ActionPos = 0;
     DoingArg = 0;
     if (UndoPtr > Cursor)
@@ -1215,7 +1222,7 @@ e_unassigned(c)
     int c;
 {				/* bound to keys that arn't really assigned */
     USE(c);
-    Beep();
+    SoundBeep();
     flush();
     return(CC_NORM);
 }
@@ -1378,7 +1385,7 @@ v_zero(c)			/* command mode 0 for vi */
     }
     else {			/* else starting an argument */
 	Cursor = InputBuf;
-	if (ActionFlag & DELETE) {
+	if (ActionFlag & TCSHOP_DELETE) {
 	   c_delfini();
 	   return(CC_REFRESH);
         }
@@ -1529,7 +1536,7 @@ e_up_hist(c)
     Char    beep = 0;
 
     USE(c);
-    UndoAction = NOP;
+    UndoAction = TCSHOP_NOP;
     *LastChar = '\0';		/* just in case */
 
     if (Hist_num == 0) {	/* save the current buffer away */
@@ -1557,7 +1564,7 @@ e_down_hist(c)
     int c;
 {
     USE(c);
-    UndoAction = NOP;
+    UndoAction = TCSHOP_NOP;
     *LastChar = '\0';		/* just in case */
 
     Hist_num -= Argument;
@@ -1618,8 +1625,8 @@ e_up_search_hist(c)
     bool    found = 0;
 
     USE(c);
-    ActionFlag = NOP;
-    UndoAction = NOP;
+    ActionFlag = TCSHOP_NOP;
+    UndoAction = TCSHOP_NOP;
     *LastChar = '\0';		/* just in case */
     if (Hist_num < 0) {
 #ifdef DEBUG_EDIT
@@ -1685,8 +1692,8 @@ e_down_search_hist(c)
     bool    found = 0;
 
     USE(c);
-    ActionFlag = NOP;
-    UndoAction = NOP;
+    ActionFlag = TCSHOP_NOP;
+    UndoAction = TCSHOP_NOP;
     *LastChar = '\0';		/* just in case */
 
     if (Hist_num == 0)
@@ -2270,7 +2277,7 @@ e_toend(c)
     USE(c);
     Cursor = LastChar;
     if (VImode)
-	if (ActionFlag & DELETE) {
+	if (ActionFlag & TCSHOP_DELETE) {
 	    c_delfini();
 	    return(CC_REFRESH);
 	}
@@ -2289,7 +2296,7 @@ e_tobeg(c)
     if (VImode) {
        while (Isspace(*Cursor)) /* We want FIRST non space character */
 	Cursor++;
-	if (ActionFlag & DELETE) {
+	if (ActionFlag & TCSHOP_DELETE) {
 	    c_delfini();
 	    return(CC_REFRESH);
 	}
@@ -2420,24 +2427,19 @@ e_charswitch(cc)
     register Char c;
 
     USE(cc);
-    if (Cursor < LastChar) {
-	if (LastChar <= &InputBuf[1]) {
-	    return(CC_ERROR);
-	}
-	else {
-	    Cursor++;
-	}
-    }
-    if (Cursor > &InputBuf[1]) {/* must have at least two chars entered */
-	c = Cursor[-2];
-	Cursor[-2] = Cursor[-1];
-	Cursor[-1] = c;
-	return(CC_REFRESH);
-    }
-    else {
-	Cursor--;		/* Restore cursor position */
+
+    /* do nothing if we are at beginning of line or have only one char */
+    if (Cursor == &InputBuf[0] || LastChar == &InputBuf[1]) {
 	return(CC_ERROR);
     }
+
+    if (Cursor < LastChar) {
+	Cursor++;
+    }
+    c = Cursor[-2];
+    Cursor[-2] = Cursor[-1];
+    Cursor[-1] = c;
+    return(CC_REFRESH);
 }
 
 /*ARGSUSED*/
@@ -2471,7 +2473,7 @@ e_charback(c)
 	    Cursor = InputBuf;
 
 	if (VImode)
-	    if (ActionFlag & DELETE) {
+	    if (ActionFlag & TCSHOP_DELETE) {
 		c_delfini();
 		return(CC_REFRESH);
 	    }
@@ -2496,7 +2498,7 @@ v_wordback(c)
 
     Cursor = c_preword(Cursor, InputBuf, Argument); /* bounds check */
 
-    if (ActionFlag & DELETE) {
+    if (ActionFlag & TCSHOP_DELETE) {
 	c_delfini();
 	return(CC_REFRESH);
     }
@@ -2518,7 +2520,7 @@ e_wordback(c)
     Cursor = c_prev_word(Cursor, InputBuf, Argument); /* bounds check */
 
     if (VImode) 
-	if (ActionFlag & DELETE) {
+	if (ActionFlag & TCSHOP_DELETE) {
 	    c_delfini();
 	    return(CC_REFRESH);
 	}
@@ -2539,7 +2541,7 @@ e_charfwd(c)
 	    Cursor = LastChar;
 
 	if (VImode)
-	    if (ActionFlag & DELETE) {
+	    if (ActionFlag & TCSHOP_DELETE) {
 		c_delfini();
 		return(CC_REFRESH);
 	    }
@@ -2565,7 +2567,7 @@ e_wordfwd(c)
     Cursor = c_next_word(Cursor, LastChar, Argument);
 
     if (VImode)
-	if (ActionFlag & DELETE) {
+	if (ActionFlag & TCSHOP_DELETE) {
 	    c_delfini();
 	    return(CC_REFRESH);
 	}
@@ -2587,7 +2589,7 @@ v_wordfwd(c)
     Cursor = c_nexword(Cursor, LastChar, Argument);
 
     if (VImode)
-	if (ActionFlag & DELETE) {
+	if (ActionFlag & TCSHOP_DELETE) {
 	    c_delfini();
 	    return(CC_REFRESH);
 	}
@@ -2611,7 +2613,7 @@ v_wordbegnext(c)
 	Cursor++;
 
     if (VImode)
-	if (ActionFlag & DELETE) {
+	if (ActionFlag & TCSHOP_DELETE) {
 	    c_delfini();
 	    return(CC_REFRESH);
 	}
@@ -2668,7 +2670,7 @@ v_csearch_back(ch, count, tflag)
 
     Cursor = cp;
 
-    if (ActionFlag & DELETE) {
+    if (ActionFlag & TCSHOP_DELETE) {
 	Cursor++;
 	c_delfini();
 	return(CC_REFRESH);
@@ -2700,7 +2702,7 @@ v_csearch_fwd(ch, count, tflag)
 
     Cursor = cp;
 
-    if (ActionFlag & DELETE) {
+    if (ActionFlag & TCSHOP_DELETE) {
 	Cursor++;
 	c_delfini();
 	return(CC_REFRESH);
@@ -2716,8 +2718,8 @@ v_action(c)
 {
     register Char *cp, *kp;
 
-    if (ActionFlag == DELETE) {
-	ActionFlag = NOP;
+    if (ActionFlag == TCSHOP_DELETE) {
+	ActionFlag = TCSHOP_NOP;
 	ActionPos = 0;
 	
 	UndoSize = 0;
@@ -2727,17 +2729,17 @@ v_action(c)
 	    UndoSize++;
 	}
 		
-	UndoAction = INSERT;
+	UndoAction = TCSHOP_INSERT;
 	UndoPtr  = InputBuf;
 	LastChar = InputBuf;
 	Cursor   = InputBuf;
-	if (c & INSERT)
+	if (c & TCSHOP_INSERT)
 	    c_alternativ_key_map(0);
 	    
 	return(CC_REFRESH);
     }
 #ifdef notdef
-    else if (ActionFlag == NOP) {
+    else if (ActionFlag == TCSHOP_NOP) {
 #endif
 	ActionPos = Cursor;
 	ActionFlag = c;
@@ -2933,7 +2935,7 @@ v_insbeg(c)
     InsertPos = Cursor;
 
     UndoPtr  = Cursor;
-    UndoAction = DELETE;
+    UndoAction = TCSHOP_DELETE;
 
     RefCursor();		/* move the cursor */
     c_alternativ_key_map(0);
@@ -2948,7 +2950,7 @@ v_replone(c)
     USE(c);
     c_alternativ_key_map(0);
     inputmode = MODE_REPLACE_1;
-    UndoAction = CHANGE;	/* Set Up for VI undo command */
+    UndoAction = TCSHOP_CHANGE;	/* Set Up for VI undo command */
     UndoPtr = Cursor;
     UndoSize = 0;
     return(CC_NORM);
@@ -2962,7 +2964,7 @@ v_replmode(c)
     USE(c);
     c_alternativ_key_map(0);
     inputmode = MODE_REPLACE;
-    UndoAction = CHANGE;	/* Set Up for VI undo command */
+    UndoAction = TCSHOP_CHANGE;	/* Set Up for VI undo command */
     UndoPtr = Cursor;
     UndoSize = 0;
     return(CC_NORM);
@@ -3011,7 +3013,7 @@ v_insert(c)
 
     InsertPos = Cursor;
     UndoPtr = Cursor;
-    UndoAction = DELETE;
+    UndoAction = TCSHOP_DELETE;
 
     return(CC_NORM);
 }
@@ -3033,7 +3035,7 @@ v_add(c)
 
     InsertPos = Cursor;
     UndoPtr = Cursor;
-    UndoAction = DELETE;
+    UndoAction = TCSHOP_DELETE;
 
     return(CC_NORM);
 }
@@ -3049,7 +3051,7 @@ v_addend(c)
 
     InsertPos = LastChar;	/* Mark where insertion begins */
     UndoPtr = LastChar;
-    UndoAction = DELETE;
+    UndoAction = TCSHOP_DELETE;
 
     RefCursor();
     return(CC_NORM);
@@ -3064,7 +3066,11 @@ v_change_case(cc)
 
     USE(cc);
     if (Cursor < LastChar) {
+#ifndef WINNT
 	c = *Cursor;
+#else
+	c = CHAR & *Cursor;
+#endif /* WINNT */
 	if (Isupper(c))
 	    *Cursor++ = Tolower(c);
 	else if (Islower(c))
@@ -3134,16 +3140,16 @@ e_tty_int(c)
     int c;
 {			
     USE(c);
-#ifdef _MINIX
+#if defined(_MINIX) || defined(WINNT)
     /* SAK PATCH: erase all of current line, start again */
     ResetInLine(0);		/* reset the input pointers */
     xputchar('\n');
     ClearDisp();
     return (CC_REFRESH);
-#else /* !_MINIX */
+#else /* !_MINIX && !WINNT */
     /* do no editing */
     return (CC_NORM);
-#endif /* _MINIX */
+#endif /* _MINIX || WINNT */
 }
 
 /*
@@ -3321,7 +3327,11 @@ e_load_average(c)
     USE(c);
     PastBottom();
 #ifdef TIOCSTAT
-    if (ioctl(SHIN, TIOCSTAT, 0) < 0) 
+    /*
+     * Here we pass &c to the ioctl because some os's (NetBSD) expect it
+     * there even if they don't use it. (lukem@netbsd.org)
+     */
+    if (ioctl(SHIN, TIOCSTAT, (ioctl_t) &c) < 0) 
 #endif
 	xprintf(CGETS(5, 1, "Load average unavailable\n"));
     return(CC_REFRESH);
@@ -3337,7 +3347,7 @@ v_chgmeta(c)
      * Delete with insert == change: first we delete and then we leave in
      * insert mode.
      */
-    return(v_action(DELETE|INSERT));
+    return(v_action(TCSHOP_DELETE|TCSHOP_INSERT));
 }
 
 /*ARGSUSED*/
@@ -3346,7 +3356,7 @@ v_delmeta(c)
     int c;
 {
     USE(c);
-    return(v_action(DELETE));
+    return(v_action(TCSHOP_DELETE));
 }
 
 
@@ -3362,7 +3372,7 @@ v_endword(c)
 
     Cursor = c_endword(Cursor, LastChar, Argument);
 
-    if (ActionFlag & DELETE)
+    if (ActionFlag & TCSHOP_DELETE)
     {
 	Cursor++;
 	c_delfini();
@@ -3385,7 +3395,7 @@ v_eword(c)
 
     Cursor = c_eword(Cursor, LastChar, Argument);
 
-    if (ActionFlag & DELETE) {
+    if (ActionFlag & TCSHOP_DELETE) {
 	Cursor++;
 	c_delfini();
 	return(CC_REFRESH);
@@ -3497,8 +3507,8 @@ v_undo(c)
 
     USE(c);
     switch (UndoAction) {
-    case DELETE|INSERT:
-    case DELETE:
+    case TCSHOP_DELETE|TCSHOP_INSERT:
+    case TCSHOP_DELETE:
 	if (UndoSize == 0) return(CC_NORM);
 	cp = UndoPtr;
 	kp = UndoBuf;
@@ -3511,10 +3521,10 @@ v_undo(c)
 	LastChar -= UndoSize;
 	Cursor   =  UndoPtr;
 	
-	UndoAction = INSERT;
+	UndoAction = TCSHOP_INSERT;
 	break;
 
-    case INSERT:
+    case TCSHOP_INSERT:
 	if (UndoSize == 0) return(CC_NORM);
 	cp = UndoPtr;
 	Cursor = UndoPtr;
@@ -3523,10 +3533,10 @@ v_undo(c)
 	for (loop = 0; loop < UndoSize; loop++)	/* copy the chars */
 	    *cp++ = *kp++;
 
-	UndoAction = DELETE;
+	UndoAction = TCSHOP_DELETE;
 	break;
 
-    case CHANGE:
+    case TCSHOP_CHANGE:
 	if (UndoSize == 0) return(CC_NORM);
 	cp = UndoPtr;
 	Cursor = UndoPtr;
