@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/ed.term.c,v 1.19 1996/04/26 19:18:16 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/ed.term.c,v 1.21 1998/04/08 13:58:35 christos Exp $ */
 /*
  * ed.term.c: Low level terminal interface
  */
@@ -35,8 +35,9 @@
  * SUCH DAMAGE.
  */
 #include "sh.h"
+#ifndef WINNT
 
-RCSID("$Id: ed.term.c,v 1.19 1996/04/26 19:18:16 christos Exp $")
+RCSID("$Id: ed.term.c,v 1.21 1998/04/08 13:58:35 christos Exp $")
 
 #include "ed.h"
 #include "ed.term.h"
@@ -1115,3 +1116,11 @@ tty_printchar(s)
     xputchar('\n');
 }
 #endif /* DEBUG_TTY */
+#else /* WINNT */
+int
+tty_cooked_mode(td)
+    void *td;
+{
+    return do_nt_check_cooked_mode()
+}
+#endif /* !WINNT */
