@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/gethost.c,v 1.6 1996/10/11 17:32:08 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/gethost.c,v 1.7 1997/10/28 22:34:19 christos Exp $ */
 /*
  * gethost.c: Create version file from prototype
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: gethost.c,v 1.6 1996/10/11 17:32:08 christos Exp $")
+RCSID("$Id: gethost.c,v 1.7 1997/10/28 22:34:19 christos Exp $")
 
 #ifdef SCO
 # define perror __perror
@@ -99,6 +99,11 @@ static const char *keyword[] =
 #define S_CODE		2
 #define S_KEYWORD	3
 
+static int findtoken __P((char *));
+static char *gettoken __P((char **, char  *));
+
+int main __P((int, char *[]));
+
 /* findtoken():
  *	Return the token number of the given token
  */
