@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/sh.dir.c,v 3.42 1996/06/22 21:44:28 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/sh.dir.c,v 3.44 1997/10/27 22:44:25 christos Exp $ */
 /*
  * sh.dir.c: Directory manipulation functions
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.dir.c,v 3.42 1996/06/22 21:44:28 christos Exp $")
+RCSID("$Id: sh.dir.c,v 3.44 1997/10/27 22:44:25 christos Exp $")
 
 /*
  * C Shell - directory management
@@ -347,7 +347,7 @@ dnormalize(cp, exp)
 	 * If the path starts with a slash, we are not relative to
 	 * the current working directory.
 	 */
-	if ( *start == '/' )
+	if (ABSOLUTEP(start))
 	    *cwd = '\0';
 # ifdef apollo
 	slashslash = cwd[0] == '/' && cwd[1] == '/';
@@ -486,7 +486,8 @@ dgoto(cp)
 {
     Char   *dp;
 
-    if (*cp != '/') {
+    if (!ABSOLUTEP(cp))
+    {
 	register Char *p, *q;
 	int     cwdlen;
 
@@ -512,7 +513,11 @@ dgoto(cp)
     else
 	dp = cp;
 
+#ifdef WINNT
+    cp = SAVE(getwd(NULL));
+#else /* !WINNT */
     cp = dcanon(cp, dp);
+#endif /* WINNT */
     return cp;
 }
 
@@ -820,7 +825,7 @@ dcanon(cp, p)
      * christos: if the path given does not start with a slash prepend cwd. If
      * cwd does not start with a slash or the result would be too long abort().
      */
-    if (*cp != '/') {
+    if (!ABSOLUTEP(cp)) {
 	Char    tmpdir[MAXPATHLEN];
 
 	p1 = varval(STRcwd);
@@ -1310,6 +1315,7 @@ recdirs(fname, def)
     struct directory *dp;
     unsigned int    num;
     Char   *snum;
+    Char    qname[MAXPATHLEN*2];
 
     if (fname == NULL && !def) 
 	return;
@@ -1345,10 +1351,10 @@ recdirs(fname, def)
 
 	if (cdflag == 0) {
 	    cdflag = 1;
-	    xprintf("cd '%S'\n", dp->di_name);
+	    xprintf("cd %S\n", quote_meta(qname, dp->di_name));
 	}
 	else
-	    xprintf("pushd '%S'\n", dp->di_name);
+	    xprintf("pushd %S\n", quote_meta(qname, dp->di_name));
 
 	if (num-- == 0)
 	    break;
