@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/sh.func.c,v 3.66 1996/10/05 17:39:10 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/sh.func.c,v 3.72 1998/04/08 17:57:25 christos Exp $ */
 /*
  * sh.func.c: csh builtin functions
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.func.c,v 3.66 1996/10/05 17:39:10 christos Exp $")
+RCSID("$Id: sh.func.c,v 3.72 1998/04/08 17:57:25 christos Exp $")
 
 #include "ed.h"
 #include "tw.h"
@@ -63,6 +63,7 @@ static	void	search		__P((int, int, Char *));
 static	int	getword		__P((Char *));
 static	void	toend		__P((void));
 static	void	xecho		__P((int, Char **));
+static	bool	islocale_var	__P((Char *));
 
 struct biltins *
 isbfunc(t)
@@ -106,8 +107,8 @@ isbfunc(t)
 	int i;
 
 	bp = bp1 + ((bp2 - bp1) >> 1);
-	if ((i = *cp - *bp->bname) == 0 &&
-	    (i = Strcmp(cp, str2short(bp->bname))) == 0)
+	if ((i = ((char) *cp) - *bp->bname) == 0 &&
+	    (i = StrQcmp(cp, str2short(bp->bname))) == 0)
 	    return bp;
 	if (i < 0)
 	    bp2 = bp;
@@ -332,6 +333,9 @@ dologin(v, c)
     struct command *c;
 {
     USE(c);
+#ifdef WINNT
+    USE(v);
+#else /* !WINNT */
     islogin();
     rechist(NULL, adrof(STRsavehist) != NULL);
     (void) signal(SIGTERM, parterm);
@@ -339,6 +343,7 @@ dologin(v, c)
     (void) execl(_PATH_USRBIN_LOGIN, "login", short2str(v[1]), NULL);
     untty();
     xexit(1);
+#endif /* !WINNT */
 }
 
 
@@ -1212,8 +1217,9 @@ done:
 }
 
 /* check whether an environment variable should invoke 'set_locale()' */
-static bool islocale_var(var)
-register Char *var;
+static bool
+islocale_var(var)
+    Char *var;
 {
     static Char *locale_vars[] = {
 	STRLANG,	STRLC_CTYPE,	STRLC_NUMERIC,	STRLC_TIME,
@@ -1312,11 +1318,13 @@ dosetenv(v, c)
 # ifdef LC_COLLATE
 	(void) setlocale(LC_COLLATE, "");
 # endif
-# if defined(NLS_CATALOGS) && defined(LC_MESSAGES)
+# ifdef NLS_CATALOGS
+#  ifdef LC_MESSAGES
 	(void) setlocale(LC_MESSAGES, "");
+#  endif /* LC_MESSAGES */
 	(void) catclose(catd);
 	nlsinit();
-# endif /* NLS_CATALOGS && LC_MESSAGES */
+# endif /* NLS_CATALOGS */
 # ifdef LC_CTYPE
 	(void) setlocale(LC_CTYPE, ""); /* for iscntrl */
 # endif /* LC_CTYPE */
@@ -1346,7 +1354,18 @@ dosetenv(v, c)
 	xfree((ptr_t) lp);
 	return;
     }
-
+#ifdef WINNT
+    if (eq(vp, STRtcshlang)) {
+	nlsinit();
+	xfree((ptr_t) lp);
+	return;
+    }
+    if (eq(vp, STRNTonlystartexes)) {
+	__nt_only_start_exes = 1;
+	xfree((ptr_t) lp);
+	return;
+    }
+#endif /* WINNT */
     if (eq(vp, STRKTERM)) {
 	char *t;
 	set(STRterm, quote(lp), VAR_READWRITE);	/* lp memory used here */
@@ -1475,11 +1494,13 @@ dounsetenv(v, c)
 # ifdef LC_COLLATE
 		    (void) setlocale(LC_COLLATE, "");
 # endif
-# if defined(NLS_CATALOGS) && defined(LC_MESSAGES)
+# ifdef NLS_CATALOGS
+#  ifdef LC_MESSAGES
 		    (void) setlocale(LC_MESSAGES, "");
+#  endif /* LC_MESSAGES */
 		    (void) catclose(catd);
 		    nlsinit();
-# endif /* NLS_CATALOGS && LC_MESSAGES */
+# endif /* NLS_CATALOGS */
 # ifdef LC_CTYPE
 	(void) setlocale(LC_CTYPE, ""); /* for iscntrl */
 # endif /* LC_CTYPE */
@@ -1503,6 +1524,15 @@ dounsetenv(v, c)
 			ed_InitNLSMaps();
 
 		}
+#ifdef WINNT
+		else if (eq(name,(STRtcshlang))) {
+		    nls_dll_unload();
+		    nlsinit();
+		}
+		else if (eq(name,(STRNTonlystartexes))) {
+			__nt_only_start_exes = 0;
+		}
+#endif /* WINNT */
 		/*
 		 * start again cause the environment changes
 		 */
@@ -1536,6 +1566,9 @@ tsetenv(name, val)
     Char   *blk[2];
     Char  **oep = ep;
 
+#ifdef WINNT
+	nt_set_env(name,val);
+#endif /* WINNT */
     for (; *ep; ep++) {
 	for (cp = name, dp = *ep; *cp && (*cp & TRIM) == *dp; cp++, dp++)
 	    continue;
@@ -1627,11 +1660,15 @@ doumask(v, c)
 #   define toset(a) ((a) + 1)
 #  endif /* aiws */
 # else /* BSDLIMIT */
-#  if defined(BSD4_4) && !defined(__386BSD__) && !defined(bsdi)
+#  if defined(BSD4_4) && !defined(__386BSD__)
     typedef quad_t RLIM_TYPE;
 #  else
-    typedef unsigned long RLIM_TYPE;
-#  endif /* BSD4_4 && !__386BSD__ && !defined(bsdi) */
+#   if defined(SOLARIS2)
+     typedef rlim_t RLIM_TYPE;
+#   else
+     typedef unsigned long RLIM_TYPE;
+#   endif /* SOLARIS2 */
+#  endif /* BSD4_4 && !__386BSD__  */
 # endif /* BSDLIMIT */
 
 # if (HPUXVERSION > 700) && defined(BSDLIMIT)
@@ -1649,6 +1686,12 @@ doumask(v, c)
 #  ifndef RLIM_INFINITY
 #   define RLIM_INFINITY	0x7fffffff
 #  endif /* RLIM_INFINITY */
+   /*
+    * old versions of HP/UX counted limits in 512 bytes
+    */
+#  ifndef SIGRTMIN
+#   define FILESIZE512
+#  endif /* SIGRTMIN */
 # endif /* (HPUXVERSION > 700) && BSDLIMIT */
 
 # if SYSVREL > 3 && defined(BSDLIMIT)
@@ -1941,7 +1984,7 @@ plim(lp, hard)
     limit = hard ? rlim.rlim_max : rlim.rlim_cur;
 # endif /* BSDLIMIT */
 
-# if !defined(BSDLIMIT) || defined(hpux)
+# if !defined(BSDLIMIT) || defined(FILESIZE512)
     /*
      * Christos: filesize comes in 512 blocks. we divide by 2 to get 1024
      * blocks. Note we cannot pre-multiply cause we might overflow (A/UX)
@@ -1952,7 +1995,7 @@ plim(lp, hard)
 	else
 	    div = (div == 1024 ? 2 : 1);
     }
-# endif /* !BSDLIMIT || hpux */
+# endif /* !BSDLIMIT || FILESIZE512 */
 
     if (limit == RLIM_INFINITY)
 	xprintf("unlimited");
@@ -2020,11 +2063,11 @@ setlim(lp, hard, limit)
 
     (void) getrlimit(lp->limconst, &rlim);
 
-#  ifdef hpux
+#  ifdef FILESIZE512
     /* Even though hpux has setrlimit(), it expects fsize in 512 byte blocks */
     if (limit != RLIM_INFINITY && lp->limconst == RLIMIT_FSIZE)
 	limit /= 512;
-#  endif /* hpux */
+#  endif /* FILESIZE512 */
     if (hard)
 	rlim.rlim_max = limit;
     else if (limit == RLIM_INFINITY && euid != 0)
@@ -2062,7 +2105,7 @@ dosuspend(v, c)
 #ifdef BSDJOBS
     int     ctpgrp;
 
-    sigret_t (*old) ();
+    signalfun_t old;
 #endif /* BSDJOBS */
     
     USE(c);
@@ -2271,6 +2314,9 @@ nlsinit()
 #ifdef NLS_CATALOGS
     catd = catopen("tcsh", MCLoadBySet);
 #endif
+#ifdef WINNT
+    nls_dll_init();
+#endif /* WINNT */
     errinit();		/* init the errorlist in correct locale */
     mesginit();		/* init the messages for signals */
     dateinit();		/* init the messages for dates */
