@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/sh.lex.c,v 3.46 1996/10/05 17:39:12 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/sh.lex.c,v 3.49 1998/04/08 13:58:54 christos Exp $ */
 /*
  * sh.lex.c: Lexical analysis into tokens
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.lex.c,v 3.46 1996/10/05 17:39:12 christos Exp $")
+RCSID("$Id: sh.lex.c,v 3.49 1998/04/08 13:58:54 christos Exp $")
 
 #include "ed.h"
 /* #define DEBUG_INP */
@@ -754,6 +754,7 @@ getexcl(sc)
 	uselastevent = 0;
 	lastev = eventno;
     }
+    else
 	lastev = eventno;
     hp = gethent(sc);
     if (hp == 0)
@@ -1563,30 +1564,36 @@ reread:
 #endif /* BSDJOBS */
 	c = bgetc();
 	if (c < 0) {
-#ifndef POSIX
-# ifdef TERMIO
+#ifndef WINNT
+# ifndef POSIX
+#  ifdef TERMIO
 	    struct termio tty;
-# else /* SGTTYB */
+#  else /* SGTTYB */
 	    struct sgttyb tty;
-# endif /* TERMIO */
-#else /* POSIX */
+#  endif /* TERMIO */
+# else /* POSIX */
 	    struct termios tty;
-#endif /* POSIX */
+# endif /* POSIX */
+#endif /* !WINNT */
 	    if (wanteof)
 		return (-1);
 	    /* was isatty but raw with ignoreeof yields problems */
-#ifndef POSIX
-# ifdef TERMIO
+#ifndef WINNT
+# ifndef POSIX
+#  ifdef TERMIO
 	    if (ioctl(SHIN, TCGETA, (ioctl_t) & tty) == 0 &&
 		(tty.c_lflag & ICANON))
-# else /* GSTTYB */
+#  else /* GSTTYB */
 	    if (ioctl(SHIN, TIOCGETP, (ioctl_t) & tty) == 0 &&
 		(tty.sg_flags & RAW) == 0)
-# endif /* TERMIO */
-#else /* POSIX */
+#  endif /* TERMIO */
+# else /* POSIX */
 	    if (tcgetattr(SHIN, &tty) == 0 &&
 		(tty.c_lflag & ICANON))
-#endif /* POSIX */
+# endif /* POSIX */
+#else /* WINNT */
+	    if (isatty(SHIN))
+#endif /* !WINNT */
 	    {
 		/* was 'short' for FILEC */
 #ifdef BSDJOBS
@@ -1710,8 +1717,15 @@ bgetc()
 		fbuf[0][i] = (unsigned char) tbuf[i];
 	    feobp += c;
 	}
+#ifndef WINNT
 	c = fbuf[0][fseekp - fbobp];
 	fseekp++;
+#else
+	do {
+	    c = fbuf[0][fseekp - fbobp];
+	    fseekp++;
+	} while(c == '\r');
+#endif /* !WINNT */
 	return (c);
     }
 
@@ -1749,8 +1763,15 @@ bgetc()
 	if (c == 0 || (c < 0 && fixio(SHIN, errno) == -1))
 	    return (-1);
     }
+#ifndef WINNT
     c = fbuf[(int) fseekp / BUFSIZE][(int) fseekp % BUFSIZE];
     fseekp++;
+#else
+    do {
+	c = fbuf[(int) fseekp / BUFSIZE][(int) fseekp % BUFSIZE];
+	fseekp++;
+    } while(c == '\r');
+#endif /* !WINNT */
     return (c);
 }
 
