@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/sh.print.c,v 3.8 1996/04/26 19:20:06 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/sh.print.c,v 3.11 1998/04/08 17:57:30 christos Exp $ */
 /*
  * sh.print.c: Primitive Output routines.
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.print.c,v 3.8 1996/04/26 19:20:06 christos Exp $")
+RCSID("$Id: sh.print.c,v 3.11 1998/04/08 17:57:30 christos Exp $")
 
 #include "ed.h"
 
@@ -196,6 +196,7 @@ flush()
 {
     register int unit;
     static int interrupted = 0;
+    size_t sz;
 
     /* int lmode; */
 
@@ -223,7 +224,11 @@ flush()
     }
 #endif
 #endif
-    (void) write(unit, linbuf, (size_t) (linp - linbuf));
+    sz = (size_t) (linp - linbuf);
+    if (write(unit, linbuf, sz) != sz && !haderr)
+	/* XXX: This ends up to cause an infinite loop if we lose our tty */
+	stderror(ERR_SILENT);
+
     linp = linbuf;
     interrupted = 0;
 }
