@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/tc.alloc.c,v 3.30 1996/04/26 19:20:34 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/tc.alloc.c,v 3.32 1998/04/08 13:59:01 christos Exp $ */
 /*
  * tc.alloc.c (Caltech) 2/21/82
  * Chris Kingsley, kingsley@cit-20.
@@ -44,13 +44,20 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.alloc.c,v 3.30 1996/04/26 19:20:34 christos Exp $")
+RCSID("$Id: tc.alloc.c,v 3.32 1998/04/08 13:59:01 christos Exp $")
 
 static char   *memtop = NULL;		/* PWP: top of current memory */
 static char   *membot = NULL;		/* PWP: bottom of allocatable memory */
 
 int dont_free = 0;
 
+#ifdef WINNT
+# define malloc		fmalloc
+# define free		ffree
+# define calloc		fcalloc
+# define realloc	frealloc
+#endif /* WINNT */
+
 #ifndef SYSMALLOC
 
 #undef RCHECK
@@ -122,13 +129,13 @@ union overhead {
  * precedes the data area returned to the user.
  */
 #define	NBUCKETS ((sizeof(long) << 3) - 3)
-static union overhead *nextf[NBUCKETS];
+static union overhead *nextf[NBUCKETS] IZERO_STRUCT;
 
 /*
  * nmalloc[i] is the difference between the number of mallocs and frees
  * for a given block size.
  */
-static U_int nmalloc[NBUCKETS];
+static U_int nmalloc[NBUCKETS] IZERO_STRUCT;
 
 #ifndef lint
 static	int	findbucket	__P((union overhead *, int));
