@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/tc.const.c,v 3.39 1996/10/05 17:39:15 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/tc.const.c,v 3.44 1998/04/08 13:59:04 christos Exp $ */
 /*
  * sh.const.c: String constants for tcsh.
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.const.c,v 3.39 1996/10/05 17:39:15 christos Exp $")
+RCSID("$Id: tc.const.c,v 3.44 1998/04/08 13:59:04 christos Exp $")
 
 Char STRlogout[]	= { 'l', 'o', 'g', 'o', 'u', 't', '\0' };
 Char STRautologout[]	= { 'a', 'u', 't', 'o', 'l', 'o', 'g', 'o', 'u', 't', 
@@ -93,6 +93,7 @@ Char STRpath[]		= { 'p', 'a', 't', 'h', '\0' };
 Char STRprompt[]	= { 'p', 'r', 'o', 'm', 'p', 't', '\0' };
 Char STRprompt2[]	= { 'p', 'r', 'o', 'm', 'p', 't', '2', '\0' };
 Char STRprompt3[]	= { 'p', 'r', 'o', 'm', 'p', 't', '3', '\0' };
+Char STRrprompt[]	= { 'r', 'p', 'r', 'o', 'm', 'p', 't', '\0' };
 Char STRellipsis[]	= { 'e', 'l', 'l', 'i', 'p', 's', 'i', 's', '\0' };
 Char STRcwd[]		= { 'c', 'w', 'd', '\0' };
 Char STRowd[]		= { 'o', 'w', 'd', '\0' };
@@ -138,6 +139,7 @@ Char STRprecmd[]	= { 'p', 'r', 'e', 'c', 'm', 'd', '\0' };
 Char STRcwdcmd[]	= { 'c', 'w', 'd', 'c', 'm', 'd', '\0' };
 Char STRperiodic[]	= { 'p', 'e', 'r', 'i', 'o', 'd', 'i', 'c', '\0' };
 Char STRtperiod[]	= { 't', 'p', 'e', 'r', 'i', 'o', 'd', '\0' };
+Char STRmf[]		= { '-', 'f', '\0' };
 Char STRml[]		= { '-', 'l', '\0' };
 Char STRslash[]		= { '/', '\0' };
 Char STRdotsl[]		= { '.', '/', '\0' };
@@ -252,6 +254,19 @@ Char STRerase[]		= { 'e', 'r', 'a', 's', 'e', '\0' };
 Char STRprev[]		= { 'p', 'r', 'e', 'v', '\0' };
 Char STRcomplete[]	= { 'c', 'o', 'm', 'p', 'l', 'e', 't', 'e', '\0' };
 Char STRenhance[]	= { 'e', 'n', 'h', 'a', 'n', 'c', 'e', '\0' };
+#ifdef WINNT
+Char STRigncase[]	= { 'i', 'g', 'n', 'c', 'a', 's', 'e', '\0' };
+Char STRoldtitle[]	= { 'o', 'l', 'd', 't', 'i', 't', 'l', 'e', '\0' };
+Char STRNTlamepathfix[] = { 'N', 'T', 'l', 'a', 'm', 'e', 'p', 'a', 't', 'h',
+			    'f', 'i','x','\0'};
+Char STRNTlameargfix[]  = { 'N', 'T', 'l', 'a', 'm', 'e', 'a', 'r', 'g', 'f',
+			    'i', 'x', '\0' };
+Char STRtcshlang[]	= { 'T', 'C', 'S', 'H', 'L', 'A', 'N', 'G', '\0' };
+Char STRtcshsubsthb[]	= { 'T', 'C', 'S', 'H', 'S', 'U', 'B', 'S', 'T',
+			    'H', 'B', '\0' };
+Char STRNTonlystartexes[]={ 'N', 'T', 'O', 'N', 'L', 'Y', 'S', 'T', 'A', 'R',
+			    'T', 'E', 'X', 'E', 'S', '\0' };
+#endif /* WINNT */
 Char STRautoexpand[]	= { 'a', 'u', 't', 'o', 'e', 'x', 'p', 'a', 'n', 'd',
 			    '\0' };
 Char STRautocorrect[]	= { 'a', 'u', 't', 'o', 'c', 'o', 'r', 'r', 'e', 'c',
@@ -327,6 +342,10 @@ Char STRsys53[] 	= { 's', 'y', 's', '5', '.', '3', '\0' };
 Char STRver[]		= { 'v', 'e', 'r', '\0' };
 #endif /* apollo */
 
+#ifdef _OSD_POSIX
+Char STRwarnebcdic[]    = { 'w', 'a', 'r', 'n', 'e', 'b', 'c', 'd', 'i', 'c', '\0' };
+#endif
+
 Char STRup[]		= { 'u', 'p', '\0' };
 Char STRdown[]		= { 'd', 'o', 'w', 'n', '\0' };
 Char STRleft[]		= { 'l', 'e', 'f', 't', '\0' };
