@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/tc.os.h,v 3.66 1996/10/05 17:39:17 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/tc.os.h,v 3.72 1998/04/08 17:57:33 christos Exp $ */
 /*
  * tc.os.h: Shell os dependent defines
  */
@@ -37,7 +37,9 @@
 #ifndef _h_tc_os
 #define _h_tc_os
 
+#ifndef WINNT
 #define NEEDstrerror		/* Too hard to find which systems have it */
+#endif /* WINNT */
 
 
 #ifdef notdef 
@@ -60,8 +62,11 @@
 # include <limits.h>
 #endif /* atp vmsposix */
 
-#ifdef DECOSF1
+#if defined(DECOSF1) || defined(HPUXVERSION)
 # include <sys/signal.h>
+#endif /* DECOSF1 || HPUXVERSION */
+
+#ifdef DECOSF1
 # include <sys/ioctl.h>
 #endif /* DECOSF1 */
 
@@ -144,9 +149,10 @@ struct ucred {
 #  define CSUSP 032
 # endif	/* CSUSP */
 
+# include <signal.h>
 # if !defined(hp9000s500) && !(defined(SIGRTMAX) || defined(SIGRTMIN))
 /*
- * hpux < 7 || hpux >= 10
+ * hpux < 7
  */
 #  include <sys/bsdtty.h>
 # endif /* !hp9000s500 && !(SIGRTMAX || SIGRTMIN) */
@@ -227,15 +233,19 @@ struct ucred {
  *
  * From: scott@craycos.com (Scott Bolte)
  */
-#ifdef F_SETFD
-# define close_on_exec(fd, v) fcntl((fd), F_SETFD, v)
-#else /* !F_SETFD */
-# ifdef FIOCLEX
-# define close_on_exec(fd, v) ioctl((fd), ((v) ? FIOCLEX : FIONCLEX), NULL)
-# else /* !FIOCLEX */
-# define close_on_exec(fd, v)	/* Nothing */
-# endif /* FIOCLEX */
-#endif /* F_SETFD */
+#ifndef WINNT
+# ifdef F_SETFD
+#  define close_on_exec(fd, v) fcntl((fd), F_SETFD, v)
+# else /* !F_SETFD */
+#  ifdef FIOCLEX
+#   define close_on_exec(fd, v) ioctl((fd), ((v) ? FIOCLEX : FIONCLEX), NULL)
+#  else /* !FIOCLEX */
+#   define close_on_exec(fd, v)	/* Nothing */
+#  endif /* FIOCLEX */
+# endif /* F_SETFD */
+#else /* WINNT */
+# define close_on_exec(fd, v) nt_close_on_exec((fd),(v))
+#endif /* !WINNT */
 
 /*
  * Stat
@@ -433,9 +443,9 @@ struct ucred {
 # define NEEDgethostname
 #endif /* _SEQUENT_ */
 
-#if defined(BSD) && defined(POSIXJOBS) && !defined(__hp_osf)
+#if defined(BSD) && defined(POSIXJOBS) && !defined(BSD4_4) && !defined(__hp_osf)
 # define setpgid(pid, pgrp)	setpgrp(pid, pgrp)
-#endif /* BSD && POSIXJOBS && !__hp_osf */
+#endif /* BSD && POSIXJOBS && && !BSD4_4 && !__hp_osf */
 
 #if defined(BSDJOBS) && !(defined(POSIX) && defined(POSIXJOBS))
 # if !defined(_AIX370) && !defined(_AIXPS2)
@@ -484,11 +494,11 @@ struct ucred {
 #ifndef POSIX
 # define mygetpgrp()    getpgrp(0)
 #else /* POSIX */
-# if defined(BSD) || defined(SUNOS4) || defined(IRIS4D) || defined(DGUX)
+# if (defined(BSD) && !defined(BSD4_4)) || defined(SUNOS4) || defined(IRIS4D) || defined(DGUX) || defined(HPRT)
 #  define mygetpgrp()    getpgrp(0)
-# else /* !(BSD || SUNOS4 || IRIS4D || DGUX) */
+# else /* !((BSD && !BSD4_4) || SUNOS4 || IRIS4D || DGUX || HPRT) */
 #  define mygetpgrp()    getpgrp()
-# endif	/* BSD || SUNOS4 || IRISD || DGUX */
+# endif	/* (BSD && BSD4_4) || SUNOS4 || IRISD || DGUX  || HPRT */
 #endif /* POSIX */
 
 
@@ -521,7 +531,7 @@ typedef struct timeval timeval_t;
 #endif /* NeXT */
 
 #ifndef NEEDgethostname
-extern int gethostname();
+extern int gethostname __P((char *, int));
 #endif /* NEEDgethostname */
 
 #if !defined(POSIX) || defined(SUNOS4) || defined(UTekV) || defined(sysV88)
@@ -544,17 +554,19 @@ extern caddr_t sbrk __P((int));
 extern int qsort();
 #  endif /* SYSVREL == 0 && !__lucid */
 # else /* !SUNOS4 */
-#  ifndef hpux
-#   if __GNUC__ != 2
+#  ifndef WINNT
+#   ifndef hpux
+#    if __GNUC__ != 2
 extern int abort();
-#   endif /* __GNUC__ != 2 */
-#   ifndef fps500
+#    endif /* __GNUC__ != 2 */
+#    ifndef fps500
 extern int qsort();
-#   endif /* !fps500 */
-#  else /* !hpux */
+#    endif /* !fps500 */
+#   else /* !hpux */
 extern void abort();
 extern void qsort();
-#  endif /* hpux */
+#   endif /* hpux */
+#  endif /* !WINNT */
 # endif	/* SUNOS4 */
 #ifndef _CX_UX
 extern void perror();
@@ -682,16 +694,16 @@ extern int getpeername __P((int, struct sockaddr *, int *));
 # endif /* REMOTEHOST */
 #endif /* SUNOS4 && __GNUC__ == 2 */
 
-#if (defined(BSD) && !defined(__386BSD__)) || defined(SUNOS4) 
+#if (defined(BSD) && !defined(BSD4_4)) || defined(SUNOS4) 
 # if defined(__alpha) && defined(__osf__) && DECOSF1 < 200
 extern void bcopy	__P((const void *, void *, size_t));
 #  define memmove(a, b, c) (bcopy((char *) (b), (char *) (a), (int) (c)), a)
 # endif /* __alpha && __osf__ && DECOSF1 < 200 */
-#endif /* (BSD && !__386BSD__) || SUNOS4 */
+#endif /* (BSD && !BSD4_4) || SUNOS4 */
 
-#if !defined(hpux) && !defined(COHERENT) && ((SYSVREL < 4) || defined(_SEQUENT_)) && !defined(__386BSD__) && !defined(memmove)
+#if !defined(hpux) && !defined(COHERENT) && ((SYSVREL < 4) || defined(_SEQUENT_)) && !defined(BSD4_4) && !defined(memmove)
 # define NEEDmemmove
-#endif /* !hpux && !COHERENT && (SYSVREL < 4 || _SEQUENT_) && !__386BSD__ && !memmove */
+#endif /* !hpux && !COHERENT && (SYSVREL < 4 || _SEQUENT_) && !BSD4_4 && !memmove */
 
 #if defined(UTek) || defined(pyr)
 # define NEEDmemset
@@ -703,18 +715,23 @@ extern void bcopy	__P((const void *, void *, size_t));
 
 #if SYSVREL == 4
 # ifdef REMOTEHOST
-extern int getpeername();
+/* Irix6 defines getpeername(int, void *, int *) which conflicts with
+   the definition below. */
+#  if !defined(__sgi)
+struct sockaddr;
+extern int getpeername __P((int, struct sockaddr *, int *));
+#  endif /* __sgi */
 # endif /* REMOTEHOST */
 # ifndef BSDTIMES
-extern int getrlimit();
-extern int setrlimit();
+extern int getrlimit __P((int, struct rlimit *));
+extern int setrlimit __P((int, const struct rlimit *));
 # endif /* !BSDTIMES */
 # if !defined(IRIS4D) && !defined(SOLARIS2)
 extern int wait3();	/* I think some bizarre systems still need this */
 # endif /* !IRIS4D && !SOLARIS2 */
 # if defined(SOLARIS2)
 #  undef NEEDstrerror
-extern char *strerror();
+extern char *strerror __P((int));
 # endif /* SOLARIS2 */
 #endif /* SYSVREL == 4 */
 
