@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/tc.prompt.c,v 3.24 1996/04/26 19:21:21 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/tc.prompt.c,v 3.30 1998/04/08 17:57:35 christos Exp $ */
 /*
  * tc.prompt.c: Prompt printing stuff
  */
@@ -36,17 +36,20 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.prompt.c,v 3.24 1996/04/26 19:21:21 christos Exp $")
+RCSID("$Id: tc.prompt.c,v 3.30 1998/04/08 17:57:35 christos Exp $")
 
 #include "ed.h"
+#include "tw.h"
 
 /*
  * kfk 21oct1983 -- add @ (time) and / ($cwd) in prompt.
  * PWP 4/27/87 -- rearange for tcsh.
  * mrdch@com.tau.edu.il 6/26/89 - added ~, T and .# - rearanged to switch()
  *                 instead of if/elseif
- * Luke Mewburn, <lm@rmit.edu.au>:  6-Sep-91 - changed date format,
- *	   16-Feb-94 - rewrote directory prompt code, added $ellipsis
+ * Luke Mewburn, <lukem@connect.com.au>:
+ *	6-Sep-91	changed date format
+ *	16-Feb-94	rewrote directory prompt code, added $ellipsis
+ *	29-Dec-96	added rprompt support
  */
 
 static char   *month_list[12];
@@ -153,6 +156,21 @@ printprompt(promptno, str)
 	SetAttributes(0);
 	flush();
     }
+
+    RPromptBuf[0] = '\0';
+    if (promptno == 0) {	/* determine rprompt if using main prompt */
+	cp = varval(STRrprompt);
+	tprintf(FMT_PROMPT, RPromptBuf, cp, INBUFSIZE - 2, NULL, lclock, NULL);
+
+				/* if not editing, put rprompt after prompt */
+	if (!editing && RPromptBuf[0] != '\0') {
+	    for (cp = RPromptBuf; *cp ; )
+		(void) putraw(*cp++);
+	    SetAttributes(0);
+	    putraw(' ');
+	    flush();
+	}
+    }
 }
 
 void
@@ -167,9 +185,9 @@ tprintf(what, buf, fmt, siz, str, tim, info)
     Char   *z, *q;
     Char    attributes = 0;
     static int print_prompt_did_ding = 0;
-    const char   *cz;
+    const unsigned char   *cz;
     Char    buff[BUFSIZE];
-    char    cbuff[BUFSIZE];
+    unsigned char    cbuff[BUFSIZE];
 
     Char *p  = buf;
     Char *ep = &p[siz];
@@ -179,8 +197,8 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 			/* prompt stuff */
     static Char *olddir = NULL, *olduser = NULL;
     extern int tlength;	/* cache cleared */
-    register int updirs;
-    int pdirs;
+    size_t sz, pdirs;
+    int updirs;
 
     for (; *cp; cp++) {
 	if (p >= ep)
@@ -190,9 +208,9 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 	    switch (*cp) {
 	    case 'R':
 		if (what == FMT_HISTORY)
-		    fmthist('R', info, str = cbuff);
+		    fmthist('R', info, str = (char *) cbuff, sizeof(cbuff));
 		if (str != NULL)
-		    for (; *str; *p++ = attributes | *str++)
+		    for (; *str; *p++ = attributes | (unsigned char) *str++)
 			if (p >= ep) break;
 		break;
 	    case '#':
@@ -202,13 +220,13 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 	    case 'h':
 		switch (what) {
 		case FMT_HISTORY:
-		    fmthist('h', info, cbuff);
+		    fmthist('h', info, cbuff, sizeof(cbuff));
 		    break;
 		case FMT_SCHED:
-		    (void) xsprintf(cbuff, "%d", *(int *)info);
+		    (void) xsnprintf(cbuff, sizeof(cbuff), "%d", *(int *)info);
 		    break;
 		default:
-		    (void) xsprintf(cbuff, "%d", eventno + 1);
+		    (void) xsnprintf(cbuff, sizeof(cbuff), "%d", eventno + 1);
 		    break;
 		}
 		for (cz = cbuff; *cz; *p++ = attributes | *cz++)
@@ -288,7 +306,7 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 	    case 'M':
 #ifndef HAVENOUTMP
 		if (what == FMT_WHO)
-		    cz = who_info(info, 'M', cbuff);
+		    cz = who_info(info, 'M', cbuff, sizeof(cbuff));
 		else 
 #endif /* HAVENOUTMP */
 		    cz = getenv("HOST");
@@ -304,7 +322,7 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 	    case 'm':
 #ifndef HAVENOUTMP
 		if (what == FMT_WHO)
-		    cz = who_info(info, 'm', cbuff);
+		    cz = who_info(info, 'm', cbuff, sizeof(cbuff));
 		else 
 #endif /* HAVENOUTMP */
 		    cz = getenv("HOST");
@@ -315,7 +333,7 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 			if (p >= ep) break;
 		break;
 
-			/* lm: new directory prompt code */
+			/* lukem: new directory prompt code */
 	    case '~':
 	    case '/':
 	    case '.':
@@ -341,7 +359,12 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 			/* option to determine fixed # of dirs from path */
 		if (Scp == '.' || Scp == 'C') {
 		    int skip;
-
+#ifdef WINNT
+		    if (z[1] == ':') {
+		    	*p++ = attributes | *z++;
+		    	*p++ = attributes | *z++;
+		    }
+#endif /* WINNT */
 		    q = z;
 		    while (*z)				/* calc # of /'s */
 			if (*z++ == '/')
@@ -396,21 +419,21 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 			    *p++ = attributes | '+';
 			} else
 			    *p++ = attributes | ('0' + updirs);
-			*p++ = attributes | tcsh ? '>' : '%';
+			*p++ = attributes | '>';
 		    }
 		}
 		
 		for (; *z ; *p++ = attributes | *z++)
 		    if (p >= ep) break;
 		break;
-			/* lm: end of new directory prompt code */
+			/* lukem: end of new directory prompt code */
 
 	    case 'n':
 #ifndef HAVENOUTMP
 		if (what == FMT_WHO) {
-		    if ((cz = who_info(info, 'n', cbuff)) != NULL)
-			for (; *cz ; *p++ = attributes | *cz++)
-			    if (p >= ep) break;
+		    cz = who_info(info, 'n', cbuff, sizeof(cbuff));
+		    for (; cz && *cz ; *p++ = attributes | *cz++)
+			if (p >= ep) break;
 		}
 		else  
 #endif /* HAVENOUTMP */
@@ -423,9 +446,9 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 	    case 'l':
 #ifndef HAVENOUTMP
 		if (what == FMT_WHO) {
-		    if ((cz = who_info(info, 'l', cbuff)) != NULL)
-			for (; *cz ; *p++ = attributes | *cz++)
-			    if (p >= ep) break;
+		    cz = who_info(info, 'l', cbuff, sizeof(cbuff));
+		    for (; cz && *cz ; *p++ = attributes | *cz++)
+			if (p >= ep) break;
 		}
 		else  
 #endif /* HAVENOUTMP */
@@ -514,6 +537,10 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 		    for (; *z; *p++ = attributes | *z++)
 			if (p >= ep) break;
 		break;
+	    case '$':
+		sz = ep - p;
+		(void) expdollar(&p, &cp, &pdirs, attributes);
+		break;
 	    case '%':
 		*p++ = attributes | '%';
 		break;
@@ -534,8 +561,8 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 	    default:
 #ifndef HAVENOUTMP
 		if (*cp == 'a' && what == FMT_WHO) {
-		    cz = who_info(info, 'a', cbuff);
-		    for (; *cz; *p++ = attributes | *cz++)
+		    cz = who_info(info, 'a', cbuff, sizeof(cbuff));
+		    for (; cz && *cz; *p++ = attributes | *cz++)
 			if (p >= ep) break;
 		}
 		else 
@@ -552,9 +579,9 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 	    *p++ = attributes | parseescape(&cp);
 	else if (*cp == HIST) {	/* EGS: handle '!'s in prompts */
 	    if (what == FMT_HISTORY) 
-		fmthist('h', info, cbuff);
+		fmthist('h', info, cbuff, sizeof(cbuff));
 	    else
-		(void) xsprintf(cbuff, "%d", eventno + 1);
+		(void) xsnprintf(cbuff, sizeof(cbuff), "%d", eventno + 1);
 	    for (cz = cbuff; *cz; *p++ = attributes | *cz++)
 		if (p >= ep) break;
 	}
@@ -563,3 +590,62 @@ tprintf(what, buf, fmt, siz, str, tim, info)
     }
     *p = '\0';
 }
+
+Char *
+expdollar(dstp, srcp, spp, attr)
+    Char **dstp;
+    Char **srcp;
+    size_t *spp;
+    int	    attr;
+{
+    struct varent *vp;
+    Char var[MAXVARLEN];
+    Char *src = *srcp, *val;
+    Char *dst = *dstp;
+    int i, curly = 0;
+
+    /* found a variable, expand it */
+    for (i = 0; i < MAXVARLEN; i++) {
+	var[i] = *++src & TRIM;
+	if (i == 0 && var[i] == '{') {
+	    curly = 1;
+	    var[i] = *++src & TRIM;
+	}
+	if (!alnum(var[i])) {
+	    
+	    var[i] = '\0';
+	    break;
+	}
+    }
+    if (curly && (*src & TRIM) == '}')
+	src++;
+
+    vp = adrof(var);
+    val = (!vp) ? tgetenv(var) : NULL;
+    if (vp) {
+	for (i = 0; vp->vec[i] != NULL; i++) {
+	    for (val = vp->vec[i]; *spp > 0 && *val; (*spp)--)
+		*dst++ = *val++ | attr;
+	    if (vp->vec[i+1] && *spp > 0) {
+		*dst++ = ' ' | attr;
+		(*spp)--;
+	    }
+	}
+    }
+    else if (val) {
+	for (; *spp > 0 && *val; (*spp)--)
+	    *dst++ = *val++ | attr;
+    }
+    else {
+	**dstp = '\0';
+	*srcp = src;
+	return NULL;
+    }
+    *dst = '\0';
+
+    val = *dstp;
+    *srcp = src;
+    *dstp = dst;
+
+    return val;
+}
