@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/tc.vers.c,v 3.42 1996/04/26 19:21:45 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/tc.vers.c,v 3.43 1997/10/27 22:44:38 christos Exp $ */
 /*
  * tc.vers.c: Version dependent stuff
  */
@@ -37,7 +37,7 @@
 #include "sh.h"
 #include "tw.h"
 
-RCSID("$Id: tc.vers.c,v 3.42 1996/04/26 19:21:45 christos Exp $")
+RCSID("$Id: tc.vers.c,v 3.43 1997/10/27 22:44:38 christos Exp $")
 
 #include "patchlevel.h"
 
@@ -141,13 +141,14 @@ fix_version()
 	ostype = STRunknown;
 
 
-    (void) xsprintf(version,
+    (void) xsnprintf(version, sizeof(version),
 "tcsh %d.%.2d.%.2d (%s) %s (%S-%S-%S) options %s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
 	     REV, VERS, PATCHLEVEL, ORIGIN, DATE, machtype, vendor, ostype,
 	     SSSTR, NLSSTR, LFSTR, DLSTR, VISTR, DTRSTR, BYESTR,
 	     ALSTR, KANSTR, SMSTR, HBSTR, NGSTR, RHSTR, AFSSTR, NDSTR,
 	     LOCALSTR);
     set(STRversion, SAVE(version), VAR_READWRITE);
-    (void) xsprintf(version, "%d.%.2d.%.2d", REV, VERS, PATCHLEVEL);
+    (void) xsnprintf(version, sizeof(version), "%d.%.2d.%.2d",
+		     REV, VERS, PATCHLEVEL);
     set(STRtcsh, SAVE(version), VAR_READWRITE);
 }
