@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/tw.init.c,v 3.19 1996/04/26 19:23:08 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/tw.init.c,v 3.23 1998/04/08 17:57:43 christos Exp $ */
 /*
  * tw.init.c: Handle lists of things to complete
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tw.init.c,v 3.19 1996/04/26 19:23:08 christos Exp $")
+RCSID("$Id: tw.init.c,v 3.23 1998/04/08 17:57:43 christos Exp $")
 
 #include "tw.h"
 #include "ed.h"
@@ -214,12 +214,7 @@ tw_cmd_add(name)
 {
     int len;
 
-    if (name[0] == '#' || name[0] == '.')	/* emacs temp's, .files	*/
-	return;
     len = (int) Strlen(name) + 2;
-    if (name[len - 3] == '~')			/* No emacs backups */
-	return;
-    
     (void) Strcpy(tw_str_add(&tw_cmd, len), name);
 } /* end tw_cmd_add */
 
@@ -247,6 +242,7 @@ tw_cmd_cmd()
     register Char **pv;
     struct varent *v = adrof(STRpath);
     struct varent *recexec = adrof(STRrecognize_only_executables);
+    int len;
 
 
     if (v == NULL) /* if no path */
@@ -268,7 +264,13 @@ tw_cmd_cmd()
 	    name = str2short(dp->d_name);
 	    if (dp->d_ino == 0 || (recexec && !executable(dir, name, 0)))
 		continue;
-	    tw_cmd_add(name);
+            len = (int) Strlen(name) + 2;
+            if (name[0] == '#' ||	/* emacs temp files	*/
+		name[0] == '.' ||	/* .files		*/
+		name[len - 3] == '~' ||	/* emacs backups	*/
+		name[len - 3] == '%')	/* textedit backups	*/
+                continue;		/* Ignore!		*/
+            tw_cmd_add(name);
 	}
 	(void) closedir(dirp);
 	if (recexec)
@@ -627,9 +629,9 @@ tw_logname_start(dfd, pat)
 {
     USE(pat);
     SETDIR(dfd)
-#ifndef _VMS_POSIX
+#if !defined(_VMS_POSIX) && !defined(WINNT)
     (void) setpwent();	/* Open passwd file */
-#endif /* atp vmsposix */
+#endif /* !_VMS_POSIX && !WINNT */
 } /* end tw_logname_start */
 
 
@@ -653,10 +655,12 @@ tw_logname_next(dir, flags)
     USE(flags);
     USE(dir);
     TW_HOLD();
-#ifndef _VMS_POSIX
+#if !defined(_VMS_POSIX) && !defined(WINNT)
     /* ISC does not declare getpwent()? */
     pw = (struct passwd *) getpwent();
-#endif /* atp vmsposix */
+#else /* _VMS_POSIX || WINNT */
+    pw = NULL;
+#endif /* !_VMS_POSIX && !WINNT */
     TW_RELS();
 
     if (pw == NULL) {
@@ -679,9 +683,9 @@ tw_logname_end()
 #ifdef YPBUGS
     fix_yp_bugs();
 #endif
-#ifndef _VMS_POSIX
+#if !defined(_VMS_POSIX) && !defined(WINNT)
    (void) endpwent();
-#endif /* atp vmsposix */
+#endif /* !_VMS_POSIX && !WINNT */
 } /* end tw_logname_end */
 
 
@@ -696,9 +700,9 @@ tw_grpname_start(dfd, pat)
 {
     USE(pat);
     SETDIR(dfd)
-#ifndef _VMS_POSIX
+#if !defined(_VMS_POSIX) && !defined(_OSD_POSIX) && !defined(WINNT)
     (void) setgrent();	/* Open group file */
-#endif /* atp vmsposix */
+#endif /* !_VMS_POSIX && !_OSD_POSIX && !WINNT */
 } /* end tw_grpname_start */
 
 
@@ -722,9 +726,11 @@ tw_grpname_next(dir, flags)
     USE(flags);
     USE(dir);
     TW_HOLD();
-#ifndef _VMS_POSIX
+#if !defined(_VMS_POSIX) && !defined(_OSD_POSIX) && !defined(WINNT)
     gr = (struct group *) getgrent();
-#endif /* atp vmsposix */
+#else /* _VMS_POSIX || _OSD_POSIX || WINNT */
+    gr = NULL;
+#endif /* !_VMS_POSIX && !_OSD_POSIX && !WINNT */
     TW_RELS();
 
     if (gr == NULL) {
@@ -747,9 +753,9 @@ tw_grpname_end()
 #ifdef YPBUGS
     fix_yp_bugs();
 #endif
-#ifndef _VMS_POSIX
+#if !defined(_VMS_POSIX) && !defined(_OSD_POSIX) && !defined(WINNT)
    (void) endgrent();
-#endif /* atp vmsposix */
+#endif /* !_VMS_POSIX && !_OSD_POSIX && !WINNT */
 } /* end tw_grpname_end */
 
 /* tw_file_start():
