@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/tw.parse.c,v 3.76 1996/10/05 17:39:20 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/tw.parse.c,v 3.82 1998/04/08 17:57:45 christos Exp $ */
 /*
  * tw.parse.c: Everyone has taken a shot in this futile effort to
  *	       lexically analyze a csh line... Well we cannot good
@@ -39,7 +39,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tw.parse.c,v 3.76 1996/10/05 17:39:20 christos Exp $")
+RCSID("$Id: tw.parse.c,v 3.82 1998/04/08 17:57:45 christos Exp $")
 
 #include "tw.h"
 #include "ed.h"
@@ -112,7 +112,7 @@ static	Char	 filetype		__P((Char *, Char *));
 static	int	 t_glob			__P((Char ***, int));
 static	int	 c_glob			__P((Char ***));
 static	int	 is_prefix		__P((Char *, Char *));
-static	int	 is_prefixmatch		__P((Char *, Char *));
+static	int	 is_prefixmatch		__P((Char *, Char *, int));
 static	int	 is_suffix		__P((Char *, Char *));
 static	int	 recognize		__P((Char *, Char *, int, int, int));
 static	int	 ignored		__P((Char *));
@@ -620,6 +620,7 @@ is_prefix(check, template)
     return (TRUE);
 } /* end is_prefix */
 
+
 /* is_prefixmatch():
  *	return true if check matches initial chars in template
  *	This differs from PWB imatch in that if check is null
@@ -627,10 +628,11 @@ is_prefix(check, template)
  * and matches on shortening of commands
  */
 static int
-is_prefixmatch(check, template)
-    register Char *check, *template;
+is_prefixmatch(check, template, igncase)
+    Char *check, *template;
+    int igncase;
 {
-    Char MCH1,MCH2;
+    Char MCH1, MCH2;
 
     for (; *check; check++, template++) {
 	if ((*check & TRIM) != (*template & TRIM)) {
@@ -639,7 +641,9 @@ is_prefixmatch(check, template)
             MCH1 = Isupper(MCH1) ? Tolower(MCH1) : MCH1;
             MCH2 = Isupper(MCH2) ? Tolower(MCH2) : MCH2;
             if (MCH1 != MCH2) {
-                if ((*check & TRIM) == '-' || (*check & TRIM) == '.' || (*check & TRIM) == '_') {
+                if (!igncase && ((*check & TRIM) == '-' || 
+				 (*check & TRIM) == '.' ||
+				 (*check & TRIM) == '_')) {
                     MCH1 = MCH2 = (*check & TRIM);
                     if (MCH1 == '_') {
                         MCH2 = '-';
@@ -679,9 +683,7 @@ is_suffix(check, template)
     for (;;) {
 	if (t == template)
 	    return 1;
-	--t;
-	--c;
-	if (c == check || (*t & TRIM) != (*c & TRIM))
+	if (c == check || (*--t & TRIM) != (*--c & TRIM))
 	    return 0;
     }
 } /* end is_suffix */
@@ -796,12 +798,22 @@ recognize(exp_name, item, name_length, numitems, enhanced)
     Char MCH1, MCH2;
     register Char *x, *ent;
     register int len = 0;
+#ifdef WINNT
+    struct varent *vp;
+    int igncase;
+    igncase = (vp = adrof(STRcomplete)) != NULL &&
+	Strcmp(*(vp->vec), STRigncase) == 0;
+#endif /* WINNT */
 
     if (numitems == 1) {	/* 1st match */
 	copyn(exp_name, item, MAXNAMLEN);
 	return (0);
     }
-    if (!enhanced) {
+    if (!enhanced
+#ifdef WINNT
+	&& !igncase
+#endif /* WINNT */
+    ) {
 	for (x = exp_name, ent = item; *x && (*x & TRIM) == (*ent & TRIM); x++, ent++)
 	    len++;
     } else {
@@ -858,6 +870,9 @@ tw_collect_items(command, looking, exp_dir, exp_name, target, pat, flags)
     struct varent *vp;
     int len, enhanced;
     int cnt = 0;
+#ifdef WINNT
+    int igncase;
+#endif /* WINNT */
 
 
     flags = 0;
@@ -953,9 +968,17 @@ tw_collect_items(command, looking, exp_dir, exp_name, target, pat, flags)
 	case RECOGNIZE_ALL:
 	case RECOGNIZE_SCROLL:
 
+#ifdef WINNT
+ 	    igncase = (vp = adrof(STRcomplete)) != NULL && 
+		Strcmp(*(vp->vec), STRigncase) == 0;
+#endif /* WINNT */
 	    enhanced = (vp = adrof(STRcomplete)) != NULL && !Strcmp(*(vp->vec),STRenhance);
-	    if (enhanced) {
-	        if (!is_prefixmatch(target, item)) 
+	    if (enhanced
+#ifdef WINNT
+		|| igncase
+#endif /* WINNT */
+	    ) {
+	        if (!is_prefixmatch(target, item, 0)) 
 		    break;
      	    } else {
 	        if (!is_prefix(target, item)) 
@@ -971,8 +994,12 @@ tw_collect_items(command, looking, exp_dir, exp_name, target, pat, flags)
 	    if (text_check && isadirectory(exp_dir, item))
 		break;
 
-	    if (gpat && !Gmatch(item, pat))
-		if (!text_check && !dir_check && !isadirectory(exp_dir, item))
+	    /*
+	     * Only pattern match directories if we're checking
+	     * for directories.
+	     */
+	    if (gpat && !Gmatch(item, pat) &&
+		(dir_check || !isadirectory(exp_dir, item)))
 		    break;
 
 	    /*
@@ -1671,6 +1698,10 @@ extract_dir_and_name(path, dir, name)
     register Char *p;
 
     p = Strrchr(path, '/');
+#ifdef WINNT
+    if (p == NULL)
+	p = Strrchr(path, ':');
+#endif /* WINNT */
     if (p == NULL) {
 	copyn(name, path, MAXNAMLEN);
 	dir[0] = '\0';
@@ -1691,10 +1722,8 @@ Char *
 dollar(new, old)
     Char   *new, *old;
 {
-    Char    var[MAXVARLEN];
-    Char   *val, *p;
-    int     space;
-    int	    i;
+    Char    *p;
+    size_t   space;
 
     for (space = FILSIZ, p = new; *old && space > 0;)
 	if (*old != '$') {
@@ -1702,36 +1731,8 @@ dollar(new, old)
 	    space--;
 	}
 	else {
-	    struct varent *vp;
-
-	    /* found a variable, expand it */
-	    for (i = 0; i < MAXVARLEN; i++) {
-		var[i] = *++old & TRIM;
-		if (!alnum(var[i])) {
-		    var[i] = '\0';
-		    break;
-		}
-	    }
-	    vp = adrof(var);
-	    val = (!vp) ? tgetenv(var) : NULL;
-	    if (vp) {
-		for (i = 0; vp->vec[i] != NULL; i++) {
-		    for (val = vp->vec[i]; space > 0 && *val; space--)
-			*p++ = *val++ | QUOTE;
-		    if (vp->vec[i+1] && space > 0) {
-			*p++ = ' ';	/* | QUOTE ? */
-			space--;
-		    }
-		}
-	    }
-	    else if (val) {
-		for (;space > 0 && *val; space--)
-		    *p++ = *val++ | QUOTE;
-	    }
-	    else {
-		*new = '\0';
-		return (NULL);
-	    }
+	    if (expdollar(&p, &old, &space, QUOTE) == NULL)
+		return NULL;
 	}
     *p = '\0';
     return (new);
@@ -2028,7 +2029,7 @@ print_by_column(dir, items, count, no_file_suffix)
 
     maxwidth += no_file_suffix ? 1 : 2;	/* for the file tag and space */
     columns = TermH / maxwidth;		/* PWP: terminal size change */
-    if (!columns)
+    if (!columns || !isatty(didfds ? 1 : SHOUT))
 	columns = 1;
     rows = (count + (columns - 1)) / columns;
 
